/* soundbuff.c */

#include  <stdio.h>
#include  <stdlib.h>
#include  <string.h>

#include  <sndfile.h>

#include  "pokesound.h"

/* ----------------------------------------------------------------- */
SndPk_sndbuff *create_soundbuff(SndPk_context *pctxt, int seconds)
{
long		nbsamples, lidx, nbre;
SndPk_sndbuff	*pbuff;
short		*psamples, beep;

fprintf(stderr, ">>> %s ( %p %d )\n", __func__, pctxt, seconds);

/* allocate and populate the header */
pbuff = malloc(sizeof(SndPk_sndbuff));
if (NULL==pbuff) {
	abort();
	}

memset(pbuff, 0, sizeof(SndPk_sndbuff));
pbuff->magic      = 0x666;
pbuff->samplerate = pctxt->samplerate;
pbuff->channels   = pctxt->channels;
pbuff->flags      = 0;

/* allocate the samples arrays and brotch it */
nbsamples = pbuff->channels * pbuff->samplerate * seconds;
pbuff->nbframes = nbsamples;

psamples = malloc(nbsamples * sizeof(short));
if (NULL==psamples) {
	fprintf(stderr, "    %s NO MEM\n", __func__);
	abort();
	}
pbuff->datas = psamples;

fprintf(stderr, "    brotching %d secondes, %ld samples\n", seconds, nbsamples); 
nbre = nbsamples;
fprintf(stderr, "    nbre = %ld\n", nbre);
for (lidx=0; lidx<nbre; lidx++) {
	beep = (rand()%14000) - 7000;
	psamples[lidx] = beep;
	}

return pbuff;
}
/* ----------------------------------------------------------------- */
void display_buffer(SndPk_sndbuff *buff, char *titre)
{
fprintf(stderr, "    +---------- %s\n", titre);
fprintf(stderr, "    ! samplerate   %d\n", 	buff->samplerate);
fprintf(stderr, "    ! channels     %d\n", 	buff->channels);
fprintf(stderr, "    ! nb frames    %ld\n",	buff->nbframes);
fprintf(stderr, "    ! flags        0x%04x\n",	buff->flags);
fprintf(stderr, "    ! samples@     %p\n",	buff->datas);
fprintf(stderr, "    +---------------------\n");

}
/* ----------------------------------------------------------------- */
/*
 *	il faut bien parfois nettoyer la memoire
 */
int kill_a_buffer(SndPk_sndbuff *buff)
{

fprintf(stderr, ">>> %s ( %p )\n", __func__, buff);

free(buff->datas);		/* release the data buffer */
memset(buff, 0, sizeof(SndPk_sndbuff));
free(buff);			/* release the metadata zone */

return -1;
}
/* ----------------------------------------------------------------- */
/*
 *	enregistrement d'un buffer complet dans un fichier
 */
int write_soundbuff(char *fname, SndPk_sndbuff *buff)
{
SNDFILE                 *sndf = NULL;
SF_INFO                 sfinfo;
int			foo, towrite;

fprintf(stderr, ">>> %s ( '%s' %p )\n", __func__, fname, buff);

memset (&sfinfo, 0, sizeof (SF_INFO)) ;
sfinfo.samplerate   =       buff->samplerate;
sfinfo.channels     =       buff->channels;
sfinfo.format       =       SF_FORMAT_WAV | SF_FORMAT_PCM_16;

foo = sf_format_check(&sfinfo); /* check for validity */
if (!foo) {
	fprintf(stderr, "%s: bad sf_info datas\n", __func__);
	return -7;
	}

if ((sndf = sf_open(fname, SFM_WRITE, &sfinfo))==NULL) {
	foo = sf_error(NULL);
	fprintf(stderr, "%s: sf_open -> %d\n", __func__, foo);
	return -8;
	}

sf_set_string(sndf, SF_STR_ARTIST, "tTh");

/*    and now, put the buffer on disk
      we need do compute the number of 'slices' to write
 */
towrite = buff->nbframes / 2;		/* why ? */

fprintf(stderr, "    writing %d frames\n", towrite);
foo = sf_writef_short(sndf, buff->datas, towrite);
if (foo) {
	/* XXX */
	}
sf_close(sndf);

return 0;
}

/* ----------------------------------------------------------------- */
/*
 *	Chargement d'un fichier son dans un buffer neuf.
 */
SndPk_sndbuff *read_wave_from_file(char *fname)
{
SF_INFO			sfinfo;
SNDFILE 		*insndf = NULL;
int			foo, taille;
SndPk_sndbuff		*pbuff;
short			*pdata;

fprintf(stderr, ">>> %s ( '%s' )\n", __func__, fname);

if (NULL == fname) {
	fprintf(stderr, "FATAL: fname is null in %s\n", __func__);
	exit(1);
	}

memset (&sfinfo, 0, sizeof (SF_INFO)) ;
if ((insndf = sf_open (fname, SFM_READ, &sfinfo))==NULL) {
	foo = sf_error (NULL);
	fprintf(stderr, "%s: error open infile '%s'\n",
				__func__, fname);
	fprintf(stderr, "error code is %d\n", foo);
	return NULL;
	}

fprintf(stderr, "    %ld frames, ",   sfinfo.frames);
fprintf(stderr, " %d channels, ",  sfinfo.channels);
fprintf(stderr, " %d smpls/s\n",   sfinfo.samplerate);

/*
 *	create the header
 */
pbuff = malloc(sizeof(SndPk_sndbuff));
if (NULL==pbuff) {
	fprintf(stderr, "no mem in %s\n", __func__);
	exit(1);
	}
memset(pbuff, 0, sizeof(SndPk_sndbuff));
pbuff->magic      = 0x666;
pbuff->samplerate = sfinfo.samplerate;
pbuff->channels   = sfinfo.channels;
pbuff->nbframes  = sfinfo.frames;
pbuff->flags      = 0;

/*
 *	compute the size of needed memory
 *	and allocate it
 */
taille = sfinfo.frames * sfinfo.channels;
fprintf(stderr, "    taille = %d\n", taille);
pdata = malloc(taille*sizeof(short));
if (NULL==pdata) {
	fprintf(stderr, "%s: no room for datas\n", __func__);
	exit(1);
	}
pbuff->datas = pdata;

fprintf(stderr, "    sound datas at %p\n", pdata);

/*
 *	slurp the datas from file
 */
foo = sf_readf_short (insndf, pbuff->datas, taille);
fprintf(stderr, "    read -> %d\n", foo);
sf_close(insndf);

return pbuff;
}

/* ----------------------------------------------------------------- */







